/*
 * compile:

gcc -nostdlib -fno-zero-initialized-in-bss -fno-function-cse -fno-jump-tables -Wl,-N -fPIE ghostid.c

 * disassemble:			objdump -d a.out
 * confirm no relocation:	readelf -r a.out
 * generate executable:		objcopy -O binary a.out b.out
 *
 * and then the resultant b.out will be grub4dos executable.
 */

#include "grub4dos.h"
char mbr[512];

int i_dummy = 0x66666666;	/* this is needed, see the following comment. */

/* gcc treat the following as data only if a global initialization like the
 * above line occurs.
 */

/* a valid executable file for grub4dos must end with these 8 bytes */
asm(".long 0x03051805");
asm(".long 0xBCBAA7BA");
/* thank goodness gcc will place the above 8 bytes at the end of the b.out
 * file. Do not insert any other asm lines here.
 */

int
main ()
{
	void *p = &main;
	char *arg = p - (*(int *)(p - 8));
	int flags = (*(int *)(p - 12));

  unsigned long part = 0xFFFFFF;
  unsigned long start, len, offset, ext_offset1;
  unsigned long type, entry1;
  typedef struct part_info {unsigned long id, part, start;} partinfo;

#define RET_VAR 0x4FF00
#define MAX_PART 26

  partinfo PART_INFO[MAX_PART], *PI, *P2;
  unsigned long id;
  unsigned long i = 0xFF;
 
  if (! *arg || *arg == ' ' || *arg == '\t')
  {
	current_drive = saved_drive;
	current_partition = saved_partition;
  }
  else if (! set_device (arg))
    return 0;

  /* The drive must be a hard disk.  */
  if (! (current_drive & 0x80))
    {
      errnum = ERR_BAD_ARGUMENT;
      return 0;
    }
  
  /* The partition must be a PC slice.  */
  if ((current_partition >> 16) == 0xFF
      || (current_partition & 0xFFFF) != 0xFFFF)
    {
      errnum = ERR_BAD_ARGUMENT;
      return 0;
    }
  
  /* Look for the partition.  */
  PI = PART_INFO;
  id = 0UL;
  while ((	next_partition_drive		= current_drive,
		next_partition_dest		= 0xFFFFFF,
		next_partition_partition	= &part,
		next_partition_type		= &type,
		next_partition_start		= &start,
		next_partition_len		= &len,
		next_partition_offset		= &offset,
		next_partition_entry		= &entry1,
		next_partition_ext_offset	= &ext_offset1,
		next_partition_buf		= mbr,
		next_partition ()))
	{
	  if (type != PC_SLICE_TYPE_NONE
		&& ! IS_PC_SLICE_TYPE_BSD (type)
		&& ! IS_PC_SLICE_TYPE_EXTENDED (type))
	  {
		if (part == current_partition)
			i=id;
		PI->id = ++id;
		PI->part = part;
		PI->start = start;
		PI++;
		if (id >= MAX_PART) break;
	  }
	  errnum = ERR_NONE;
	}
	errnum = ERR_NONE;

	for (PI=PART_INFO;PI<PART_INFO+id;PI++)
	{
		for (P2=PI+1;P2<PART_INFO+id;P2++)
		{
			if (PI->start > P2->start) 
			{
				unsigned long t_id;
				t_id=P2->id; P2->id=PI->id; PI->id=t_id;
			}
		}
	}

	if (i < id)
	{
		sprintf((char *)RET_VAR,"%d:%d\r\r",(unsigned char)(current_drive - 0x7F),(unsigned char)PART_INFO[i].id);
		if (debug > 0 /*&& flags*/)
			printf(" (hd%d,%d) in Ghost Style is: %d:%d\n",(unsigned int)(current_drive-0x80),(unsigned int)(current_partition >> 16),(unsigned int)(current_drive - 0x7F),(unsigned int)PART_INFO[i].id);
		return PART_INFO[i].id;
	}
	return 0;
}



